/*!
 * vue-router-mp by @Cweili - https://github.com/Cweili/vue-router-mp
 */
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var eq = _interopDefault(require('object-equal'));
var qs = _interopDefault(require('querystringify'));

function runQueue(queue, fn, cb) {
  var step = function step(index) {
    if (index >= queue.length) {
      cb();
    } else if (queue[index]) {
      fn(queue[index], function () {
        step(index + 1);
      });
    } else {
      step(index + 1);
    }
  };

  step(0);
}

function warn(condition, message) {
  if (process.env.NODE_ENV !== 'production' && !condition && typeof console !== 'undefined') {
    console.warn("[vue-router-mp] " + message);
  }
}
function isError(err) {
  return Object.prototype.toString.call(err).indexOf('Error') > -1;
}

var noop = function noop() {};

function transitionTo(router, location, onComplete, onAbort) {
  if (onComplete === void 0) {
    onComplete = noop;
  }

  if (onAbort === void 0) {
    onAbort = noop;
  }

  var route = router.match(location);

  if (!route) {
    onAbort();
    return;
  }

  var currentRoute = router.currentRoute;

  if (currentRoute && route.fullPath === currentRoute.fullPath) {
    warn(false, 'route is not changed');
    onAbort();
    return;
  }

  var abort = function abort(err) {
    if (isError(err)) {
      warn(false, 'uncaught error during route navigation:');
      console.error(err);
    }

    onAbort(err);
  };

  var queue = router.beforeHooks.concat(route.beforeEnter);
  var pending = route;

  var iterator = function iterator(hook, next) {
    if (pending !== route) {
      abort();
      return;
    }

    try {
      hook(route, currentRoute, function (to) {
        if (to === false || isError(to)) {
          // next(false) -> abort navigation, ensure currentURL
          abort(to);
        } else if (typeof to === 'string' || typeof to === 'object' && (typeof to.path === 'string' || typeof to.name === 'string')) {
          // next('/') or next({ path: '/' }) -> redirect
          transitionTo(router, to, onComplete, onAbort);
        } else {
          // confirm transition and pass on the value
          next(to);
        }
      });
    } catch (e) {
      abort(e);
    }
  };

  runQueue(queue, iterator, function () {
    if (pending !== route) {
      abort();
      return;
    }

    pending = null; // update route

    var prev = currentRoute;
    router.currentRoute = route;
    router.afterHooks.forEach(function (hook) {
      if (hook) hook(route, prev);
    });
    onComplete(route);
  });
}

var install = (function (router) {
  return function (Vue) {
    Vue.mixin({
      onShow: function onShow() {
        var app = router.app = this.$root;
        var $mp = app.$mp;

        if ($mp) {
          var currentRoute = router.currentRoute;
          var query = $mp.query,
              _$mp$page = $mp.page,
              page = _$mp$page === void 0 ? {} : _$mp$page;
          var route = page.route;

          if (route) {
            if (route.indexOf(router.base) !== 0) {
              route = router.base + route;
            }

            if (currentRoute.path !== route || !eq(currentRoute.query, query)) {
              transitionTo(router, {
                path: route,
                query: query
              });
            }
          }
        }
      }
    });
    Object.defineProperty(Vue.prototype, '$router', {
      get: function get() {
        return router;
      }
    });
    Object.defineProperty(Vue.prototype, '$route', {
      get: function get() {
        return router.currentRoute;
      }
    });
  };
});

/* globals wx */
function push(router, location, onComplete, onAbort) {
  return transitionTo(router, location, function (route) {
    if (route) {
      var params = {
        url: decodeURIComponent(route.fullPath),
        fail: onAbort,
        success: onComplete
      };

      if (route.isTab) {
        wx.switchTab(params);
      } else if (route.reLaunch) {
        wx.reLaunch(params);
      } else if (route.replace) {
        wx.redirectTo(params);
      } else {
        wx.navigateTo(params);
      }
    }
  }, onAbort);
}
function go(delta) {
  wx.navigateBack({
    delta: delta
  });
}

function parse(path) {
  var _path$split = path.split('?'),
      p = _path$split[0],
      q = _path$split[1];

  return {
    path: p,
    query: q ? qs.parse(q) : {}
  };
}
function stringify(path, query) {
  return (path || '') + qs.stringify(query, '?');
}

function createRoute(route, location) {
  if (location === void 0) {
    location = {};
  }

  return route && {
    name: route.name,
    meta: route.meta || {},
    path: route.path || '',
    hash: '',
    query: location.query || {},
    params: {},

    get fullPath() {
      return stringify(route.path, location.query);
    },

    beforeEnter: route.beforeEnter,
    replace: location.replace,
    reLaunch: location.reLaunch,
    isTab: location.isTab || route.isTab
  };
}

function registerHook(list, fn) {
  list.push(fn);
  return function () {
    var i = list.indexOf(fn);

    if (i > -1) {
      list.splice(i, 1);
    }
  };
}

function locationObj(location) {
  return typeof location === 'string' ? {
    path: location
  } : location;
}

var pathMap = {};
var nameMap = {};
var beforeHooks = [];
var afterHooks = [];

function router(_temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      routes = _ref.routes,
      _ref$base = _ref.base,
      base = _ref$base === void 0 ? '/' : _ref$base;

  router.addRoutes(routes);
  router.base = base;
  return router;
}

router.currentRoute = createRoute({});
router.beforeHooks = beforeHooks;
router.afterHooks = afterHooks;

router.beforeEach = function (fn) {
  return registerHook(beforeHooks, fn);
};

router.afterEach = function (fn) {
  return registerHook(afterHooks, fn);
};

router.push = function (location, onComplete, onAbort) {
  location = locationObj(location);
  return push(router, location, onComplete, onAbort);
};

router.replace = function (location, onComplete, onAbort) {
  location = locationObj(location);
  location.replace = true;
  return push(router, location, onComplete, onAbort);
};

router.go = go;

router.back = function () {
  return go(-1);
};

router.forward = function () {
  return go(1);
};

router.addRoutes = function (routes) {
  if (routes === void 0) {
    routes = [];
  }

  routes.forEach(function (route) {
    pathMap[route.path] = route;

    if (route.name) {
      nameMap[route.name] = route;
    }
  });
};

router.match = function (location) {
  location = locationObj(location);
  var _location = location,
      name = _location.name,
      path = _location.path,
      query = _location.query;
  var p = path;
  var q = query;
  var matched;

  if (name) {
    matched = nameMap[name];
  } else if (path) {
    var parsed = parse(path);
    p = parsed.path;
    q = Object.assign({}, parsed.query, query);
    matched = pathMap[p];
  }

  warn(matched, 'Can\'t match any router');
  return createRoute(matched, Object.assign({}, location, {
    name: name,
    path: p,
    query: q
  }));
};

router.install = install(router);

module.exports = router;
